from abc import ABC
from typing import Tuple, Optional, Iterable

from v25.messaging.message import Message
from v25.messaging.subject import Subject


class AbstractStorage(ABC):
    def check(self, subject: Subject) -> dict:
        raise NotImplementedError

    def push(self, m: Message) -> None:
        raise NotImplementedError

    def edit(self, old: Message, new: Message):
        raise NotImplementedError

    def delete(self, m: Message):
        raise NotImplementedError

    def pull(self, pair: Tuple[Subject, Subject], params: Optional[dict] = None) -> Iterable[Message]:
        raise NotImplementedError

    def exact(self, sfrom: Subject, sto: Subject, idnonce: bytes, editnonce: Optional[bytes]) -> Optional[Message]:
        raise NotImplementedError

    def flags(self, m: Message, flags: str):
        raise NotImplementedError


class EventStorage(AbstractStorage, ABC):
    def events(self, sfrom: Subject, sto: Subject, after: Optional[bytes]) -> Iterable[Tuple[bytes, bytes]]:
        raise NotImplementedError

    def typing(self, sfrom: Subject, sto: Subject, last: float) -> float:
        raise NotImplementedError


EVENT_PUSH = 0
EVENT_EDIT = 1
EVENT_DELETE = 2


class PushStorage(EventStorage, ABC):
    def subscribe(self, subject: Subject, subscription: dict):
        raise NotImplementedError
